@@ -14,8 +14,9 @@ mod x86_64_unknown_linux_gnu;
 mod x86_64_unknown_linux_musl;
 
 use crate::{
-    cargo_env, emit_warning, env_var_to_bool, execute_command, get_cflags, out_dir,
-    requested_c_std, target, target_arch, target_os, target_vendor, CStdRequested, OutputLibType,
+    cargo_env, emit_warning, env_var_to_bool, execute_command, get_cflags, is_no_asm, option_env,
+    out_dir, requested_c_std, target, target_arch, target_env, target_os, target_vendor,
+    CStdRequested, OutputLibType,
 };
 use std::path::PathBuf;
 
@@ -96,44 +97,85 @@ impl CcBuilder {
         }
     }
 
-    fn apply_c_std(cc_build: &mut cc::Build) {
-        match requested_c_std() {
-            CStdRequested::C99 => cc_build.std("c99"),
-            _ => cc_build.std("c11"),
-        };
+    pub(crate) fn create_builder(&self) -> cc::Build {
+        let mut cc_build = cc::Build::default();
+        cc_build.out_dir(&self.out_dir).cpp(false);
+
+        let compiler = cc_build.get_compiler();
+        if compiler.is_like_gnu() || compiler.is_like_clang() {
+            cc_build.flag("-Wno-unused-parameter");
+            if target_os() == "linux"
+                || target_os().ends_with("bsd")
+                || target_env() == "gnu"
+                || target_env() == "musl"
+            {
+                cc_build.define("_XOPEN_SOURCE", "700").flag("-pthread");
+            }
+        }
+
+        self.add_includes(&mut cc_build);
+
+        cc_build
     }
 
-    fn create_builder(&self) -> cc::Build {
-        let mut cc_build = cc::Build::default();
-        cc_build
-            .out_dir(&self.out_dir)
-            .flag("-Wno-unused-parameter")
-            .cpp(false)
-            .shared_flag(false)
-            .static_flag(true);
-        CcBuilder::apply_c_std(&mut cc_build);
-        if target_os() == "linux" {
-            cc_build.define("_XOPEN_SOURCE", "700").flag("-lpthread");
+    pub(crate) fn prepare_builder(&self) -> cc::Build {
+        let mut cc_build = self.create_builder();
+        match requested_c_std() {
+            CStdRequested::C99 => {
+                cc_build.std("c99");
+            }
+            CStdRequested::C11 => {
+                cc_build.std("c11");
+            }
+            CStdRequested::None => {
+                if target_env() == "msvc" && target_arch() == "aarch64" {
+                    // clang-cl (not "clang") will be used.
+                } else if self.compiler_check("c11", "") {
+                    cc_build.std("c11");
+                } else {
+                    cc_build.std("c99");
+                }
+            }
+        };
+
+        if let Some(cc) = option_env("CC") {
+            emit_warning(&format!("CC environment variable set: {}", cc.clone()));
         }
-        if let Some(prefix) = &self.build_prefix {
-            cc_build
-                .define("BORINGSSL_IMPLEMENTATION", "1")
-                .define("BORINGSSL_PREFIX", prefix.as_str());
+        if let Some(cxx) = option_env("CXX") {
+            emit_warning(&format!("CXX environment variable set: {}", cxx.clone()));
+        }
+
+        let compiler = cc_build.get_compiler();
+        if target_arch() == "x86" && (compiler.is_like_clang() || compiler.is_like_gnu()) {
+            cc_build.flag_if_supported("-msse2");
         }
 
         let opt_level = cargo_env("OPT_LEVEL");
         match opt_level.as_str() {
-            "0" | "1" | "2" => {}
+            "0" | "1" | "2" => {
+                if is_no_asm() {
+                    emit_warning("AWS_LC_SYS_NO_ASM found. Disabling assembly code usage.");
+                    cc_build.define("OPENSSL_NO_ASM", "1");
+                }
+            }
             _ => {
-                let file_prefix_map_option =
-                    format!("-ffile-prefix-map={}=", self.manifest_dir.display());
-                if let Ok(true) = cc_build.is_flag_supported(&file_prefix_map_option) {
-                    cc_build.flag(file_prefix_map_option);
-                } else {
-                    cc_build.flag_if_supported(format!(
-                        "-fdebug-prefix-map={}=",
-                        self.manifest_dir.display()
-                    ));
+                assert!(
+                    !is_no_asm(),
+                    "AWS_LC_SYS_NO_ASM only allowed for debug builds!"
+                );
+                if compiler.is_like_gnu() || compiler.is_like_clang() {
+                    let flag = format!("-ffile-prefix-map={}=", self.manifest_dir.display());
+                    if let Ok(true) = cc_build.is_flag_supported(&flag) {
+                        emit_warning(&format!("Using flag: {}", &flag));
+                        cc_build.flag(flag);
+                    } else {
+                        emit_warning("NOTICE: Build environment source paths might be visible in release binary.");
+                        let flag = format!("-fdebug-prefix-map={}=", self.manifest_dir.display());
+                        if let Ok(true) = cc_build.is_flag_supported(&flag) {
+                            emit_warning(&format!("Using flag: {}", &flag));
+                            cc_build.flag(flag);
+                        }
+                    }
                 }
             }
         }
@@ -146,14 +188,27 @@ impl CcBuilder {
             env::set_var("CFLAGS", cflags);
         }
 
-        self.add_includes(&mut cc_build);
+        if target_os() == "macos" {
+            // This compiler error has only been seen on MacOS x86_64:
+            // ```
+            // clang: error: overriding '-mmacosx-version-min=13.7' option with '--target=x86_64-apple-macosx14.2' [-Werror,-Woverriding-t-option]
+            // ```
+            cc_build.flag_if_supported("-Wno-overriding-t-option");
+        }
+
         cc_build
     }
 
     fn add_includes(&self, cc_build: &mut cc::Build) {
+        // The order of includes matters
+        if let Some(prefix) = &self.build_prefix {
+            cc_build
+                .define("BORINGSSL_IMPLEMENTATION", "1")
+                .define("BORINGSSL_PREFIX", prefix.as_str());
+            cc_build.include(self.manifest_dir.join("generated-include"));
+        }
         cc_build
             .include(self.manifest_dir.join("include"))
-            .include(self.manifest_dir.join("generated-include"))
             .include(self.manifest_dir.join("aws-lc").join("include"))
             .include(
                 self.manifest_dir
@@ -189,14 +244,14 @@ impl CcBuilder {
     }
 
     fn build_library(&self, lib: &Library) {
-        let mut cc_build = self.create_builder();
+        let mut cc_build = self.prepare_builder();
 
         self.add_all_files(lib, &mut cc_build);
 
         for flag in lib.flags {
             cc_build.flag(flag);
         }
-        self.compiler_checks(&mut cc_build);
+        self.run_compiler_checks();
 
         if let Some(prefix) = &self.build_prefix {
             cc_build.compile(format!("{}_crypto", prefix.as_str()).as_str());
@@ -208,9 +263,11 @@ impl CcBuilder {
     // This performs basic checks of compiler capabilities and sets an appropriate flag on success.
     // This should be kept in alignment with the checks performed by AWS-LC's CMake build.
     // See: https://github.com/search?q=repo%3Aaws%2Faws-lc%20check_compiler&type=code
-    fn compiler_check(&self, cc_build: &mut cc::Build, basename: &str, flag: &str) {
-        let output_path = self.out_dir.join(format!("{basename}.o"));
-        if let Ok(()) = cc::Build::default()
+    fn compiler_check(&self, basename: &str, flag: &str) -> bool {
+        let mut ret_val = false;
+        let output_dir = self.out_dir.join(format!("out-{basename}"));
+        let mut cc_build = self.create_builder();
+        cc_build
             .file(
                 self.manifest_dir
                     .join("aws-lc")
@@ -218,13 +275,30 @@ impl CcBuilder {
                     .join("compiler_features_tests")
                     .join(format!("{basename}.c")),
             )
-            .flag("-Wno-unused-parameter")
             .warnings_into_errors(true)
-            .try_compile(output_path.as_os_str().to_str().unwrap())
-        {
-            cc_build.define(flag, "1");
+            .out_dir(&output_dir);
+
+        let compiler = cc_build.get_compiler();
+        if compiler.is_like_gnu() || compiler.is_like_clang() {
+            cc_build.flag("-Wno-unused-parameter");
         }
-        let _ = fs::remove_file(output_path);
+        let result = cc_build.try_compile_intermediates();
+
+        if result.is_ok() {
+            if !flag.is_empty() {
+                cc_build.define(flag, "1");
+            }
+            ret_val = true;
+        }
+        if fs::remove_dir_all(&output_dir).is_err() {
+            emit_warning(&format!("Failed to remove {:?}", &output_dir));
+        }
+        emit_warning(&format!(
+            "Compilation of '{basename}.c' {} - {:?}.",
+            if ret_val { "succeeded" } else { "failed" },
+            &result
+        ));
+        ret_val
     }
 
     // This checks whether the compiler contains a critical bug that causes `memcmp` to erroneously
@@ -237,6 +311,10 @@ impl CcBuilder {
         let exec_path = out_dir().join(basename);
         let memcmp_build = cc::Build::default();
         let memcmp_compiler = memcmp_build.get_compiler();
+        if !memcmp_compiler.is_like_clang() && !memcmp_compiler.is_like_gnu() {
+            // The logic below assumes a Clang or GCC compiler is in use
+            return;
+        }
         let mut memcmp_compile_args = Vec::from(memcmp_compiler.args());
         memcmp_compile_args.push(
             self.manifest_dir
@@ -291,13 +369,9 @@ impl CcBuilder {
         }
         let _ = fs::remove_file(exec_path);
     }
-    fn compiler_checks(&self, cc_build: &mut cc::Build) {
-        self.compiler_check(cc_build, "stdalign_check", "AWS_LC_STDALIGN_AVAILABLE");
-        self.compiler_check(
-            cc_build,
-            "builtin_swap_check",
-            "AWS_LC_BUILTIN_SWAP_SUPPORTED",
-        );
+    fn run_compiler_checks(&self) {
+        self.compiler_check("stdalign_check", "AWS_LC_STDALIGN_AVAILABLE");
+        self.compiler_check("builtin_swap_check", "AWS_LC_BUILTIN_SWAP_SUPPORTED");
         self.memcmp_check();
     }
 }
@@ -327,4 +401,8 @@ impl crate::Builder for CcBuilder {
         self.build_library(&libcrypto);
         Ok(())
     }
+
+    fn name(&self) -> &'static str {
+        "CC"
+    }
 }
