@@ -1,6 +1,6 @@
 // Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 // SPDX-License-Identifier: Apache-2.0 OR ISC
-// Mon Sep  9 18:27:31 UTC 2024
+// Wed Dec 11 13:51:11 UTC 2024
 
 use crate::cc_builder::Library;
 
@@ -74,10 +74,6 @@ pub(super) const CRYPTO_LIBRARY: Library = Library {
         "crypto/cipher_extra/tls_cbc.c",
         "crypto/conf/conf.c",
         "crypto/crypto.c",
-        "crypto/curve25519/curve25519.c",
-        "crypto/curve25519/curve25519_nohw.c",
-        "crypto/curve25519/curve25519_s2n_bignum_asm.c",
-        "crypto/curve25519/spake25519.c",
         "crypto/decrepit/bio/base64_bio.c",
         "crypto/decrepit/blowfish/blowfish.c",
         "crypto/decrepit/cast/cast.c",
@@ -103,12 +99,13 @@ pub(super) const CRYPTO_LIBRARY: Library = Library {
         "crypto/engine/engine.c",
         "crypto/err/err.c",
         "crypto/evp_extra/evp_asn1.c",
+        "crypto/evp_extra/p_dh.c",
+        "crypto/evp_extra/p_dh_asn1.c",
+        "crypto/evp_extra/p_dsa.c",
         "crypto/evp_extra/p_dsa_asn1.c",
         "crypto/evp_extra/p_ec_asn1.c",
-        "crypto/evp_extra/p_ed25519.c",
         "crypto/evp_extra/p_ed25519_asn1.c",
         "crypto/evp_extra/p_hmac_asn1.c",
-        "crypto/evp_extra/p_kem.c",
         "crypto/evp_extra/p_kem_asn1.c",
         "crypto/evp_extra/p_methods.c",
         "crypto/evp_extra/p_rsa_asn1.c",
@@ -123,8 +120,6 @@ pub(super) const CRYPTO_LIBRARY: Library = Library {
         "crypto/fipsmodule/fips_shared_support.c",
         "crypto/hpke/hpke.c",
         "crypto/hrss/hrss.c",
-        "crypto/kem/kem.c",
-        "crypto/kem/kem_methods.c",
         "crypto/kyber/kem_kyber.c",
         "crypto/kyber/kyber1024r3_ref.c",
         "crypto/kyber/kyber512r3_ref.c",
@@ -132,10 +127,6 @@ pub(super) const CRYPTO_LIBRARY: Library = Library {
         "crypto/kyber/pqcrystals_kyber_ref_common/fips202.c",
         "crypto/lhash/lhash.c",
         "crypto/mem.c",
-        "crypto/ml_kem/ml_kem.c",
-        "crypto/ml_kem/ml_kem_1024_ipd.c",
-        "crypto/ml_kem/ml_kem_512_ipd.c",
-        "crypto/ml_kem/ml_kem_768_ipd.c",
         "crypto/obj/obj.c",
         "crypto/obj/obj_xref.c",
         "crypto/ocsp/ocsp_asn.c",
@@ -154,7 +145,10 @@ pub(super) const CRYPTO_LIBRARY: Library = Library {
         "crypto/pem/pem_pkey.c",
         "crypto/pem/pem_x509.c",
         "crypto/pem/pem_xaux.c",
+        "crypto/pkcs7/bio/cipher.c",
+        "crypto/pkcs7/bio/md.c",
         "crypto/pkcs7/pkcs7.c",
+        "crypto/pkcs7/pkcs7_asn1.c",
         "crypto/pkcs7/pkcs7_x509.c",
         "crypto/pkcs8/p5_pbev2.c",
         "crypto/pkcs8/pkcs8.c",
@@ -167,7 +161,6 @@ pub(super) const CRYPTO_LIBRARY: Library = Library {
         "crypto/rand_extra/entropy_passive.c",
         "crypto/rand_extra/forkunsafe.c",
         "crypto/rand_extra/fuchsia.c",
-        "crypto/rand_extra/pq_custom_randombytes.c",
         "crypto/rand_extra/rand_extra.c",
         "crypto/rand_extra/trusty.c",
         "crypto/rand_extra/windows.c",
@@ -180,6 +173,7 @@ pub(super) const CRYPTO_LIBRARY: Library = Library {
         "crypto/rsa_extra/rsa_print.c",
         "crypto/rsa_extra/rsassa_pss_asn1.c",
         "crypto/siphash/siphash.c",
+        "crypto/spake25519/spake25519.c",
         "crypto/stack/stack.c",
         "crypto/thread.c",
         "crypto/thread_none.c",
@@ -321,6 +315,8 @@ pub(super) const CRYPTO_LIBRARY: Library = Library {
         "third_party/s2n-bignum/arm/p384/bignum_nonzero_6.S",
         "third_party/s2n-bignum/arm/p384/bignum_sub_p384.S",
         "third_party/s2n-bignum/arm/p384/bignum_tomont_p384.S",
+        "third_party/s2n-bignum/arm/p384/p384_montjdouble.S",
+        "third_party/s2n-bignum/arm/p384/p384_montjdouble_alt.S",
         "third_party/s2n-bignum/arm/p521/bignum_add_p521.S",
         "third_party/s2n-bignum/arm/p521/bignum_fromlebytes_p521.S",
         "third_party/s2n-bignum/arm/p521/bignum_mul_p521.S",
@@ -330,5 +326,7 @@ pub(super) const CRYPTO_LIBRARY: Library = Library {
         "third_party/s2n-bignum/arm/p521/bignum_sqr_p521_alt.S",
         "third_party/s2n-bignum/arm/p521/bignum_sub_p521.S",
         "third_party/s2n-bignum/arm/p521/bignum_tolebytes_p521.S",
+        "third_party/s2n-bignum/arm/p521/p521_jdouble.S",
+        "third_party/s2n-bignum/arm/p521/p521_jdouble_alt.S",
     ],
 };
